/*
	PlastererApplet
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "plastererimpl.h"
#include "plastererdefine.h"

CPlastererImpl::CPlastererImpl( QWidget* parent )
	: QWidget( parent, NULL )
{
	m_pSlWaitDlg = NULL;

	setWallPaper();

	m_pChannel = new QCopChannel( "QPE/Plasterer", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );

	setFixedSize( 0, 0 );
}

CPlastererImpl::~CPlastererImpl()
{
	if ( m_pSlWaitDlg )
	{
		delete m_pSlWaitDlg;
	}
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;
	delete m_pSysChannel;
}

void CPlastererImpl::receiveMessage( const QCString& msg, const QByteArray& /*data*/ )
{
	// QDataStream stream( data, IO_ReadOnly );

	if ( msg == "version()")
	{
		version();
	}
	else if ( msg == "makeImages()" )
	{
		DBG_LOG( "CPlasterer::receiveMessage() > \"makeImages()\"" );
		makeImages();
	}
	else if ( msg == "reload()" )
	{
		DBG_LOG( "CPlasterer::receiveMessage() > \"reload()\"" );
		m_pSlWaitDlg = new SlDlgWait( this, "Plasterer\n now Making Images", TRUE, FALSE );
		m_pSlWaitDlg->move( QPoint( ( QPEApplication::desktop()->width() - m_pSlWaitDlg->width() ) / 2 ,
							( QPEApplication::desktop()->height() - m_pSlWaitDlg->height() ) / 2 ) );
		m_pSlWaitDlg->exec();
		makeImages( TRUE );
	}
	else if ( msg == "sweepWaitDlg()" )
	{
		if ( m_pSlWaitDlg )
		{
			delete m_pSlWaitDlg;
			m_pSlWaitDlg = NULL;
		}
	}
}

void CPlastererImpl::receiveSysMessage( const QCString& msg, const QByteArray& data )
{
	if ( msg == "rotateChanged(QString)" )
	{
		QDataStream stream( data, IO_ReadOnly );
		QString rotInf;
		stream >> rotInf;
		DBG_LOG( QString( "CPlastererImpl::receiveSysMessage() > data : %1" ).arg( rotInf ).latin1() );
		setWallPaper( ( rotInf == "Transformed:Rot270:Vga:0" ) ? WF_H_IMG : WF_V_IMG );
	}
}

void CPlastererImpl::version()
{
	QString ver = "Plasterer";
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void CPlastererImpl::setWallPaper( const QString& sImg )
{
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	if ( sImg != QString::null )
	{
		sImgPath += sImg;
	}
	else
	{
		sImgPath += SlMisc::isLandscape() ? WF_H_IMG : WF_V_IMG;
	}
	Config cfg( CNF_L );
	cfg.setGroup( SEC_WP );
	cfg.writeEntry( KEY_WFN, sImgPath );
	DBG_LOG( QString( "CPlastererImpl::setWallPaper() > ImgPath : %1" ).arg( sImgPath ).latin1() );
	
	{
		QCopEnvelope e( "QPE/System", "changedWallPaper()" );
	}
}

void CPlastererImpl::makeImages( bool bReload )
{
	DBG_LOG( "CPlastererImpl::makeImages()" );
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	QSize maxWidgetSize = QPEApplication::desktop()->size();
	if ( maxWidgetSize.height() < maxWidgetSize.width() )
	{
		maxWidgetSize = QSize( maxWidgetSize.height(), maxWidgetSize.width() );
	}

	QSize sizeWidget;
	
	sizeWidget = QSize( maxWidgetSize.width() - SlMisc::getScrollbarMetrics().width(),
						(int) ( (double)maxWidgetSize.height() * 0.80 ) );
	QImage imgV( cfg.readEntry( KEY_VFN, "" ) );
	makeImage( &imgV, cfg, sizeWidget, KEY_VFN );
	imgV.save( sImgPath + WF_V_IMG, "PNG" );

	sizeWidget = QSize( (int) ( (double) maxWidgetSize.width() * 0.80 ),
						maxWidgetSize.height() - SlMisc::getScrollbarMetrics().width() );
	QImage imgH( cfg.readEntry( KEY_HFN, "" ) );
	makeImage( &imgH, cfg, sizeWidget, KEY_HFN );
	imgH.save( sImgPath + WF_H_IMG, "PNG" );

	if ( bReload )
	{
		setWallPaper();
	}
	QMessageBox mb( "Plasterer", "reloaded...", QMessageBox::NoIcon,
					QMessageBox::NoButton, QMessageBox::NoButton, QMessageBox::NoButton );
	mb.showMaximized();
	{
		QCopEnvelope e( "QPE/Plasterer", "sweepWaitDlg()" );
	}
}

void CPlastererImpl::makeImage( QImage* img,
								Config& cfg,
								const QSize& rSizeDesktop,
								const QString keyFName )
{
	DBG_LOG( QString("CPlasterer::makeImage() > keyFName %1").arg( keyFName ).latin1() );
	bool isAlignV = ( keyFName == KEY_VFN ) ? TRUE : FALSE;

	if ( ! cfg.readBoolEntry( isAlignV ? KEY_VTM : KEY_HTM, TRUE )
		&& ( ! img->isNull() )
		&& ( img->size() != QSize( 0, 0 ) ) )
	{
		DBG_LOG( "Not Tiling Mode" );
		QSize rSize;
		if ( rSizeDesktop.height() > rSizeDesktop.width() )
		{
			rSize = isAlignV ? rSizeDesktop : QSize( rSizeDesktop.height(), rSizeDesktop.width() );
		}
		else
		{
			rSize = isAlignV ? QSize( rSizeDesktop.height(), rSizeDesktop.width() ) : rSizeDesktop;
		}
		QSize sizeScaling = img->size();

		if ( cfg.readBoolEntry( isAlignV ? KEY_VFT : KEY_HFT, FALSE ) )
		{
			DBG_LOG( "Fitting Mode" );
			if ( isAlignV )
			{
				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );

				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );
			}
			else
			{
				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );

				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );
			}

			if ( img->size() != sizeScaling )
			{
				*img = img->smoothScale( sizeScaling.width(), sizeScaling.height() );
			}
		}
		{
			QPoint drawP( ( rSize.width() - sizeScaling.width() ) / 2,
							( rSize.height() - sizeScaling.height() ) / 2 );
			QPixmap pixT;
			pixT.resize( rSize );
			QStringList tmpList = QStringList::split( ",", cfg.readEntry( isAlignV ? KEY_VBC : KEY_HBC, "0,0,0" ) );
			QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
			pixT.fill( tmpColor );
			QPainter p( &pixT );
			p.drawImage( drawP, *img );
			p.flush();
			*img = pixT.convertToImage();
		}
	}
}

